@@ -414,6 +414,9 @@ class CentralLogArchiveBuckets:
         org_id_param = t.add_parameter(
             Parameter("OrgId", Type="String", Description="The AWS Organization ID")
         )
+        t.add_parameter(
+            Parameter("AuditAccount", Type="String", Description="The audit account ID")
+        )
         t.add_parameter(
             Parameter(
                 "ReplicationRoleName",
@@ -484,13 +487,10 @@ class CentralLogArchiveBuckets:
                         ]
                     },
                     "Action": "s3:PutObject",
-                    "Resource": Sub(
-                        "arn:${AWS::Partition}:s3:::${LogsBucket}/AWSLogs/*"
-                    ),
+                    "Resource": Sub("arn:${AWS::Partition}:s3:::${LogsBucket}/*"),
                     "Condition": {
                         "StringEquals": {
                             "aws:SourceOrgId": Ref(org_id_param),
-                            "s3:x-amz-acl": "bucket-owner-full-control",
                         },
                     },
                 },
@@ -506,11 +506,6 @@ class CentralLogArchiveBuckets:
                     },
                     "Action": "s3:GetBucketAcl",
                     "Resource": Sub("arn:${AWS::Partition}:s3:::${LogsBucket}"),
-                    "Condition": {
-                        "StringEquals": {
-                            "aws:SourceOrgId": Ref(org_id_param),
-                        },
-                    },
                 },
                 {
                     "Sid": "BucketExistenceCheck",
@@ -524,11 +519,18 @@ class CentralLogArchiveBuckets:
                     },
                     "Action": "s3:ListBucket",
                     "Resource": Sub("arn:${AWS::Partition}:s3:::${LogsBucket}"),
-                    "Condition": {
-                        "StringEquals": {
-                            "aws:SourceOrgId": Ref(org_id_param),
-                        },
+                },
+                {
+                    "Sid": "AuditAccountRead",
+                    "Effect": "Allow",
+                    "Principal": {
+                        "AWS": Sub("arn:${AWS::Partition}:iam::${AuditAccount}:root"),
                     },
+                    "Action": ["s3:ListBucket", "s3:GetObject"],
+                    "Resource": [
+                        Sub("arn:${AWS::Partition}:s3:::${LogsBucket}"),
+                        Sub("arn:${AWS::Partition}:s3:::${LogsBucket}/*"),
+                    ],
                 },
             ],
             bucket_name=Sub("${LogsBucketPrefix}-${AWS::AccountId}-${AWS::Region}"),
